@@ -1,6 +1,6 @@
 /**
   FoxyProxy
-  Copyright (C) 2006-#%#% Eric H. Jung and LeahScape, Inc.
+  Copyright (C) 2006-#%#% Eric H. Jung and FoxyProxy, Inc.
   http://getfoxyproxy.org/
   eric.jung@yahoo.com
 
@@ -10,7 +10,8 @@
 **/
 if (!CI) {
   // we're not being included by foxyproxy.js
-  var CI = Components.interfaces, CC = Components.classes, CR = Components.results, fp;
+  var CI = Components.interfaces, CC = Components.classes,
+    CR = Components.results, fp;
 
   // Get attribute from node if it exists, otherwise return |def|.
   // No exceptions, no errors, no null returns.
@@ -21,7 +22,8 @@ if (!CI) {
   // Boolean version of GetSafe
   var gGetSafeAttrB = function(n, name, def) {
     n.QueryInterface(CI.nsIDOMElement);
-    return n ? (n.hasAttribute(name) ? n.getAttribute(name)=="true" : def) : def;
+    return n ? (n.hasAttribute(name) ? n.getAttribute(name) == "true" : def) :
+      def;
   };
 }
 
@@ -30,7 +32,8 @@ Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 ///////////////////////////// AutoConf class ///////////////////////
 function AutoConf(owner, fpp) {
   this.wrappedJSObject = this;
-  fp = fpp || CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;    
+  fp = fpp || CC["@leahscape.org/foxyproxy/service;1"].
+    getService().wrappedJSObject;    
   this.timer = CC["@mozilla.org/timer;1"].createInstance(CI.nsITimer);
   this.owner = owner;
   this._resolver = new fpProxyAutoConfig(this);
@@ -101,13 +104,16 @@ AutoConf.prototype = {
     req.channel.loadFlags |= CI.nsIRequest.LOAD_BYPASS_CACHE;
     req.send(null);
     if (req.status == 200 ||
-        (req.status == 0 && (url.indexOf("file://") == 0 || url.indexOf("ftp://") == 0 || url.indexOf("relative://") == 0))) {
+        (req.status == 0 && (url.indexOf("file://") == 0 ||
+        url.indexOf("ftp://") == 0 || url.indexOf("relative://") == 0))) {
       new fpProxyAutoConfig(this).init(url, req.responseText);
     }
     else throw new Error(fp.getMessage("http.error", [req.status]));
   },
 
   loadPAC : function() {
+    let autoconfMode = this.owner.autoconfMode;
+    let autoconfMessage = "";
     try {
       var req = CC["@mozilla.org/xmlextras/xmlhttprequest;1"]
         .createInstance(CI.nsIXMLHttpRequest);
@@ -117,37 +123,72 @@ AutoConf.prototype = {
       req.send(null);
     }
     catch(e) {
-      this.badPAC("pac.status.loadfailure", e);
+      if (autoconfMode === "pac") {
+        autoconfMessage = "pac.status.loadfailure2"; 
+      } else {
+        autoconfMessage = "wpad.status.loadfailure";
+      }
+      this.badPAC(autoconfMessage, e);
       return;
     }
-    if (req.status == 200 ||
-      (req.status == 0 && (this.url.indexOf("file://") == 0 || this.url.indexOf("ftp://") == 0 || this.url.indexOf("relative://") == 0))) {
-        try {
-          this._resolver.init(this.url, req.responseText);
+    if (req.status == 200 || (req.status == 0 &&
+         (this.url.indexOf("file://") == 0 || this.url.indexOf("ftp://") == 0 ||
+          this.url.indexOf("relative://") == 0))) {
+      try {
+        this._resolver.init(this.url, req.responseText);
+      }
+      catch(e) {
+        if (autoconfMode === "pac") {
+          autoconfMessage = "pac.status.error2"; 
+        } else {
+          autoconfMessage = "wpad.status.error";
         }
-        catch(e) {
-          this.badPAC("pac.status.error", e);
-          return;
-        }
-        this.loadNotification && fp.notifier.alert(fp.getMessage("pac.status"), fp.getMessage("pac.status.success", [this.owner.name]));
-        this.owner._enabled = true; // Use _enabled so we don't loop infinitely
-        //if (this.disabledDueToBadPAC) {
-          //this.disabledDueToBadPAC = false; /* reset */
-          //this.owner.fp.writeSettings();
-        //}
+        this.badPAC(autoconfMessage, e); 
+        return;
+      }
+      let autoconfMessageHelper = "";
+      if (autoconfMode === "pac") {
+          autoconfMessage = "pac.status"; 
+          autoconfMessageHelper = "pac.status.success2";
+      } else {
+          autoconfMessage = "wpad.status";
+          autoconfMessageHelper = "wpad.status.success";
+      } 
+      this.loadNotification && fp.notifier.alert(fp.getMessage(autoconfMessage),
+        fp.getMessage(autoconfMessageHelper, [this.owner.name]));
+      // Use _enabled so we don't loop infinitely 
+      this.owner._enabled = true;
+      //if (this.disabledDueToBadPAC) {
+        //this.disabledDueToBadPAC = false; /* reset */
+        //this.owner.fp.writeSettings();
+      //}
     }
     else {
-      this.badPAC("pac.status.loadfailure", new Error(fp.getMessage("http.error", [req.status])));
+      if (autoconfMode === "pac") {
+        autoconfMessage = "pac.status.loadfailure2"; 
+      } else {
+        autoconfMessage = "wpad.status.loadfailure";
+      } 
+      this.badPAC(autoconfMessage,
+        new Error(fp.getMessage("http.error", [req.status])));
     }
   },
 
   badPAC : function(r, e) {
-    //if (!this.disabledDueToBadPAC) { /* marker to try loading the PAC next time */
+    //if (!this.disabledDueToBadPAC) {
+      /* marker to try loading the PAC next time */
       //this.disabledDueToBadPAC = true;
       //this.owner.fp.writeSettings();
     //}
+    let autoconfMessage = "";
+    if (this.owner.autoconfMode === "pac") {
+      autoconfMessage = "pac.status"; 
+    } else {
+      autoconfMessage = "wpad.status";
+    } 
     var msg = fp.getMessage(r, [this.owner.name]) + "\n\n" + e.message;
-    this.errorNotification && fp.notifier.alert(fp.getMessage("pac.status"), msg);
+    this.errorNotification && fp.notifier.alert(fp.getMessage(autoconfMessage),
+      msg);
     if (this.owner.lastresort)
       this.owner.mode = "direct"; // don't disable!
     else if (this.disableOnBadPAC)
@@ -193,9 +234,15 @@ fpProxyAutoConfig.prototype = {
 
     /** throws a localized Error object on error */
     init: function(pacURI, pacText) {
+      let autoconfMessage = "";
       if (pacURI == "" || pacText == "") {
         dump("FoxyProxy: init(), pacURI or pacText empty\n");    
-        throw new Error(fp.getMessage("pac.empty"));
+        if (this.owner.owner.autoconfMode  === "pac") {
+          autoconfMessage = "pac.empty"; 
+        } else {
+          autoconfMessage = "wpad.empty";
+        } 
+        throw new Error(fp.getMessage(autoconfMessage));
       }
       this.sandbox = new Components.utils.Sandbox(pacURI);
       Components.utils.evalInSandbox(pacUtils, this.sandbox);
@@ -218,7 +265,12 @@ fpProxyAutoConfig.prototype = {
       // everytime in getProxyxForURI().
       if (!("FindProxyForURL" in this.sandbox)) {
         dump("FoxyProxy: init(), FindProxyForURL not found\n");
-        throw new Error(fp.getMessage("pac.fcn.notfound"));
+        if (this.owner.owner.autoconfMode === "pac") {
+          autoconfMessage = "pac.fcn.notfound2"; 
+        } else {
+          autoconfMessage = "wpad.fcn.notfound";
+        } 
+        throw new Error(fp.getMessage(autoconfMessage));
       }
       return true;
     },
