@@ -1,6 +1,6 @@
 /**
   FoxyProxy
-  Copyright (C) 2006-#%#% Eric H. Jung and LeahScape, Inc.
+  Copyright (C) 2006-#%#% Eric H. Jung and FoxyProxy, Inc.
   http://getfoxyproxy.org/
   eric.jung@yahoo.com
 
@@ -9,15 +9,23 @@
   and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 const CI = Components.interfaces, CC = Components.classes;
-var urlsTree, proxy, foxyproxy, autoconfurl, overlay, isWindows, fpc,
+var urlsTree, proxy, foxyproxy, autoconfUrl, overlay, isWindows, fpc,
+  autoconfMode, reloadFreq, loadNotification, errorNotification, autoReload,
   oldMatches = [];
 
 function onLoad() {
-  isWindows = CC["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULRuntime).OS == "WINNT";
+  isWindows = CC["@mozilla.org/xre/app-info;1"].
+    getService(CI.nsIXULRuntime).OS == "WINNT";
   fpc = CC["@leahscape.org/foxyproxy/common;1"].getService().wrappedJSObject;
   overlay = fpc.getMostRecentWindow().foxyproxy;
-  autoconfurl = document.getElementById("autoconfurl");
-  foxyproxy = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;
+  foxyproxy = CC["@leahscape.org/foxyproxy/service;1"].
+    getService().wrappedJSObject; 
+  autoconfUrl = document.getElementById("autoconfUrl");
+  autoconfMode = document.getElementById("autoconfMode");
+  reloadFreq = document.getElementById("autoConfReloadFreq");
+  loadNotification = document.getElementById("pacLoadNotificationEnabled");
+  errorNotification = document.getElementById("pacErrorNotificationEnabled");
+  autoReload = document.getElementById("autoConfURLReloadEnabled");
   proxy = window.arguments[0].inn.proxy; 
   if (window.arguments[0].inn.torwiz) {
     document.getElementById("torwiz-broadcaster").hidden = true;
@@ -42,22 +50,38 @@ function onLoad() {
   document.getElementById("port").value = proxy.manualconf.port;
   document.getElementById("isSocks").checked = proxy.manualconf.isSocks;
   document.getElementById("socksversion").value = proxy.manualconf.socksversion;
+  document.getElementById("proxyDNS").checked = proxy.proxyDNS; 
+  autoconfMode.value = proxy.autoconfMode;
+
+  if (proxy.autoconfMode === "pac") {
+    // toggleMode("auto") is called below and it includes already a call of
+    // toggleMode("pac") (or toggleMode("wpad")). We therefore omitting it here
+    // and below in the wpad code path if the proxy mode is "auto".
+    if (proxy.mode !== "auto") {
+      toggleMode("pac");
+    }
+    loadNotification.checked = proxy.autoconf.loadNotification;
+    errorNotification.checked = proxy.autoconf.errorNotification;
+    autoReload.checked = proxy.autoconf.autoReload;
+    reloadFreq.value = proxy.autoconf.reloadFreqMins; 
+  } else if (proxy.autoconfMode === "wpad") {
+    if (proxy.mode !== "auto") {
+      toggleMode("wpad");
+    }
+    loadNotification.checked = proxy.wpad.loadNotification;
+    errorNotification.checked = proxy.wpad.errorNotification;
+    autoReload.checked = proxy.wpad.autoReload;
+    reloadFreq.value = proxy.wpad.reloadFreqMins;    
+  }
   toggleMode(proxy.mode); 
-  document.getElementById("proxyDNS").checked = proxy.proxyDNS;
-  document.getElementById("WPADReloadEnabled").checked = proxy.wpad.autoReload;
-  document.getElementById("WPADReloadFreq").value = proxy.wpad.reloadFreqMins;
-  autoconfurl.value = proxy.autoconf.url;
 
   if (proxy.lastresort) {
-    document.getElementById("default-proxy-broadcaster").setAttribute("disabled", "true");
-	  document.getElementById("proxyname").disabled =
-	  	document.getElementById("proxynotes").disabled = true;
-      document.getElementById("foxyproxy-urlpatterns-tab").hidden = true;
+    document.getElementById("default-proxy-broadcaster").
+      setAttribute("disabled", "true");
+    document.getElementById("proxyname").disabled = document.
+      getElementById("proxynotes").disabled = true;
+    document.getElementById("foxyproxy-urlpatterns-tab").hidden = true;
   }
-  document.getElementById("pacLoadNotificationEnabled").checked = proxy.autoconf.loadNotification;
-  document.getElementById("pacErrorNotificationEnabled").checked = proxy.autoconf.errorNotification;
-  document.getElementById("autoConfURLReloadEnabled").checked = proxy.autoconf.autoReload;
-  document.getElementById("autoConfReloadFreq").value = proxy.autoconf.reloadFreqMins;
   // We need to copy the matches array here in order to replace the modified
   // one with the old one if the user does not press the OK button (e.g. using
   // the Cancel button or just closing the window manually).
@@ -80,7 +104,7 @@ function onCancel() {
 }
 
 function trim(s) {
-	return s.replace(/^\s*|\s*$/g, "");
+  return s.replace(/^\s*|\s*$/g, "");
 }
 
 function onOK() {
@@ -90,53 +114,64 @@ function onOK() {
   if (!name)
     name = host ? (host + ":" + port) : foxyproxy.getMessage("new.proxy");
   var enabled = document.getElementById("proxyenabled").checked,    
-    url = trim(autoconfurl.value),
-    reloadfreq = document.getElementById("autoConfReloadFreq").value;
+    url = trim(autoconfUrl.value);
   var mode = document.getElementById("mode").value;
   if (enabled) {
-    if (mode == "auto") {
-	    if (!_checkUri())
-	    	return false;
+    if (mode === "auto") {
+      if (autoconfMode.value === "pac") {
+        if (!_checkUri())
+          return false;
+      }
+    } else if (mode === "manual") {
+      if (!host) {
+        if (!port) {
+          foxyproxy.alert(this, foxyproxy.getMessage("nohostport.3"));
+          return false;
+        }
+        foxyproxy.alert(this, foxyproxy.getMessage("nohost.3"));
+        return false;
+      } else if (!port) {
+        foxyproxy.alert(this, foxyproxy.getMessage("noport.3"));
+        return false;
+      }
     }
-    else if (mode == "manual") {
-    	if (!host) {
-    		if (!port) {
-			    foxyproxy.alert(this, foxyproxy.getMessage("nohostport.3"));
-			    return false;
-    		}
-		    foxyproxy.alert(this, foxyproxy.getMessage("nohost.3"));
-		    return false;
-    	}
-    	else if (!port) {
-		    foxyproxy.alert(this, foxyproxy.getMessage("noport.3"));
-		    return false;
-		  }
-		}
   }
 
   if (!foxyproxy.isFoxyProxySimple()) {
     // Don't do this for FoxyProxy Basic
-  	if (!hasWhite() && !foxyproxy.warnings.showWarningIfDesired(window, [window.arguments[0].inn.torwiz ?
-        "torwiz.nopatterns.3" : "no.white.patterns.3", name], "white-patterns"))
-  	  return false;
+    if (!hasWhite() && !foxyproxy.warnings.showWarningIfDesired(window,
+        [window.arguments[0].inn.torwiz ? "torwiz.nopatterns.3" :
+        "no.white.patterns.3", name], "white-patterns"))
+      return false;
   }
   
-	var isSocks = document.getElementById("isSocks").checked;
-	
-	if (fpc.isThunderbird() && !isSocks && mode == "manual" && !foxyproxy.warnings.showWarningIfDesired(window, ["socksWarning"], "socks"))
-	  return false;
+  var isSocks = document.getElementById("isSocks").checked;
+
+  if (fpc.isThunderbird() && !isSocks && mode == "manual" &&
+      !foxyproxy.warnings.showWarningIfDesired(window, ["socksWarning"],
+      "socks"))
+    return false;
 
   proxy.name = name;
   proxy.notes = document.getElementById("proxynotes").value;
   proxy.selectedTabIndex = document.getElementById("tabs").selectedIndex;
-  proxy.wpad.autoReload = document.getElementById("WPADReloadEnabled").checked;
-  proxy.wpad.reloadFreqMins = document.getElementById("WPADReloadFreq").value;
-  proxy.autoconf.url = url;
-  proxy.autoconf.loadNotification = document.getElementById("pacLoadNotificationEnabled").checked;
-  proxy.autoconf.errorNotification = document.getElementById("pacErrorNotificationEnabled").checked;
-  proxy.autoconf.autoReload = document.getElementById("autoConfURLReloadEnabled").checked;
-  proxy.autoconf.reloadFreqMins = reloadfreq;
 
+  // We assign these settings here in order to have them before PAC's are
+  // loaded.
+  if (autoconfMode.value === "pac") {
+    proxy.autoconfMode = "pac";
+    proxy.autoconf.url = url;
+    proxy.autoconf.loadNotification = loadNotification.checked;
+    proxy.autoconf.errorNotification = errorNotification.checked;
+    proxy.autoconf.autoReload = autoReload.checked;
+    proxy.autoconf.reloadFreqMins = reloadFreq.value; 
+  } else {
+    proxy.autoconfMode = "wpad";
+    proxy.wpad.loadNotification = loadNotification.checked;
+    proxy.wpad.errorNotification = errorNotification.checked;
+    proxy.wpad.autoReload = autoReload.checked;
+    proxy.wpad.reloadFreqMins = reloadFreq.value; 
+  }
   proxy.mode = mode; // set this first to control PAC loading
   proxy.enabled = enabled;
   proxy.manualconf.host = host;
@@ -168,15 +203,19 @@ function hasWhite() {
 }
 
 function _checkUri() {
-	var url = trim(autoconfurl.value);
-	if (url.indexOf("://") == -1) {
-		// User didn't specify a scheme, so assume he means file:///
-		url = url.replace(/\\/g,"/"); // replaces backslashes with forward slashes; probably not strictly necessary
-		if (url[0] != "\\" && url[0] != "/") url="/"+url; // prepend a leading slash if necessary
-		url="file:///" + (isWindows?"C:":"") + url;
-		autoconfurl.value = url; // copy back to the UI
-	}
-	try {
+  var url = trim(autoconfUrl.value);
+  if (url.indexOf("://") == -1) {
+    // User didn't specify a scheme, so assume he means file:///
+
+    // Replaces backslashes with forward slashes; probably not strictly
+    // necessary.
+    url = url.replace(/\\/g,"/");
+    // prepend a leading slash if necessary 
+    if (url[0] != "\\" && url[0] != "/") url="/"+url;
+    url="file:///" + (isWindows?"C:":"") + url;
+    autoconfUrl.value = url; // copy back to the UI
+  }
+  try {
     //return foxyproxy.newURI(url);
     return CC["@mozilla.org/network/io-service;1"]
       .getService(CI.nsIIOService).newURI(url, "UTF-8", null);
@@ -189,6 +228,9 @@ function _checkUri() {
 
 function noInternalIPs() {
   let noInternalIPsChecked;
+  let localhostRegEx = "^https?://(?:[^:@/]+(?::[^@/]+)?@)?(?:localhost|127\\.\\d+\\.\\d+\\.\\d+)(?::\\d+)?/.*";
+  let localSubRegEx = "^https?://(?:[^:@/]+(?::[^@/]+)?@)?(?:192\\.168\\.\\d+\\.\\d+|10\\.\\d+\\.\\d+\\.\\d+|172\\.(?:1[6789]|2[0-9]|3[01])\\.\\d+\\.\\d+)(?::\\d+)?/.*";
+  let localHostnameRegEx = "^https?://(?:[^:@/]+(?::[^@/]+)?@)?[\\w-]+(?::\\d+)?/.*";
   if (window.arguments[0].inn.torwiz) {
     noInternalIPsChecked = document.getElementById("fpniip").checked;
   } else {
@@ -197,31 +239,42 @@ function noInternalIPs() {
   if (noInternalIPsChecked) {
     let helper = [];
     let m = CC["@leahscape.org/foxyproxy/match;1"].createInstance().
-      wrappedJSObject; 
+      wrappedJSObject;
     m.init(true, foxyproxy.getMessage("localhost2") +
-      foxyproxy.getMessage("localhost.patterns.message"),
-      "^https?://(?:[^:@/]+(?::[^@/]+)?@)?(?:localhost|127\.\d+\.\d+\.\d+)(?::\d+)?/.*",
-      false, true, false, true, false);
+      foxyproxy.getMessage("localhost.patterns.message"), localhostRegEx, false,
+      true, false, true, false);
     helper.push(m);
     m = CC["@leahscape.org/foxyproxy/match;1"].createInstance().
-      wrappedJSObject; 
+      wrappedJSObject;
     m.init(true, foxyproxy.getMessage("localsubnets2") +
-      foxyproxy.getMessage("localhost.patterns.message"), "^https?://(?:[^:@/]+(?::[^@/]+)?@)?(?:192\.168\.\d+\.\d+|10\.\d+\.\d+\.\d+|172\.(?:1[6789]|2[0-9]|3[12]))(?::\d+)?/.*",
-      false, true, false, true, false);
+      foxyproxy.getMessage("localhost.patterns.message"), localSubRegEx, false,
+      true, false, true, false);
     helper.push(m);
     m = CC["@leahscape.org/foxyproxy/match;1"].createInstance().
-      wrappedJSObject; 
+      wrappedJSObject;
     m.init(true, foxyproxy.getMessage("localhostnames2") +
-      foxyproxy.getMessage("localhost.patterns.message"),
-      "^https?://(?:[^:@/]+(?::[^@/]+)?@)?[\w-]+(?::\d+)?/.*", false, true,
-      false, true, false);
+      foxyproxy.getMessage("localhost.patterns.message"), localHostnameRegEx,
+      false, true, false, true, false);
     helper.push(m);
-    proxy.matches = helper.concat(proxy.matches); 
-    _updateView();
+    proxy.matches = helper.concat(proxy.matches);
   } else {
-    proxy.matches = proxy.matches.slice(3);
-    _updateView();
+    // We want to delete these three patterns properly even if the user somehow
+    // sorted the tree. Therefore, we have to walk through all patterns and if
+    // we find a match we remove it from the array.
+    let i = j = 0;
+    let matchesLength = proxy.matches.length;
+    do {
+      let proxyPattern = proxy.matches[i].pattern;
+      if (proxyPattern === localhostRegEx || proxyPattern === localSubRegEx ||
+          proxyPattern === localHostnameRegEx) {
+        proxy.matches.splice(i, 1);
+      } else {
+        i++;
+      }
+      j++;
+    } while  (j < matchesLength);
   }
+  _updateView();
 }
 
 function onAddEditURLPattern(isNew) {
@@ -268,6 +321,7 @@ function _updateView() {
   // patterns.
   document.getElementById("exportURLPatternCmd").setAttribute("disabled", 
     proxy.matches.length === 0); 
+  //document.getElementById("noInternalIPs").checked = proxy.noInteralIPs;
   // Redraw the trees
   urlsTree.view = makeView();
 
@@ -420,20 +474,27 @@ function toggleMode(mode) {
   if (mode == "auto") {
     document.getElementById("autoconf-broadcaster1").
       removeAttribute("disabled");
-    document.getElementById("autoconf-broadcaster3").setAttribute("disabled",
-      "true");
     document.getElementById("socks-broadcaster").setAttribute("disabled",
       "true");
     document.getElementById("disabled-broadcaster").setAttribute("disabled",
       "true");
     document.getElementById("direct-broadcaster").removeAttribute("disabled");
     document.getElementById("proxyDNS").hidden = false;
+    // We need that here to trigger the broadcaster related code in the wpad
+    // and pac codepath below in all possible cases.
+    if (document.getElementById("autoconfMode").value === "wpad") {
+      toggleMode("wpad");
+    } else {
+      toggleMode("pac"); 
+    }
     onAutoConfUrlInput();
   } else if (mode == "direct") {
     document.getElementById("disabled-broadcaster").setAttribute("disabled",
       "true");
     document.getElementById("autoconf-broadcaster1").setAttribute("disabled",
       "true");
+    document.getElementById("autoconf-broadcaster2").setAttribute("disabled",
+      "true");
     document.getElementById("autoconf-broadcaster3").setAttribute("disabled",
       "true"); 
     document.getElementById("socks-broadcaster").setAttribute("disabled",
@@ -442,21 +503,27 @@ function toggleMode(mode) {
       "true");
     document.getElementById("proxyDNS").hidden = true;
   } else if (mode == "wpad") {
-    document.getElementById("disabled-broadcaster").setAttribute("disabled",
-      "true");
-    document.getElementById("autoconf-broadcaster1").setAttribute("disabled",
-      "true");
-    document.getElementById("autoconf-broadcaster3").
-      removeAttribute("disabled");
-    document.getElementById("socks-broadcaster").setAttribute("disabled",
+    autoconfUrl.value = proxy.wpad.url;
+    autoconfUrl.setAttribute("readonly", true);
+    // We always have a URL here, thus we can remove the attribute directly.
+    document.getElementById("autoconf-broadcaster2").
+      removeAttribute("disabled"); 
+    // We do not need the file picker either.
+    document.getElementById("autoconf-broadcaster3").setAttribute("disabled",
       "true"); 
-    document.getElementById("direct-broadcaster").removeAttribute("disabled")
-    document.getElementById("proxyDNS").hidden = false; 
-  
+  } else if (mode == "pac") {
+    autoconfUrl.value = proxy.autoconf.url;
+    autoconfUrl.removeAttribute("readonly");
+    // If we clicked on WPAD first we have to enable the file picker again.
+    document.getElementById("autoconf-broadcaster3").
+      removeAttribute("disabled"); 
+    onAutoConfUrlInput();
   } else {
     document.getElementById("disabled-broadcaster").removeAttribute("disabled");
     document.getElementById("autoconf-broadcaster1").setAttribute("disabled",
       "true");
+    document.getElementById("autoconf-broadcaster2").setAttribute("disabled",
+      "true"); 
     document.getElementById("autoconf-broadcaster3").setAttribute("disabled",
       "true"); 
     if (document.getElementById("isSocks").checked) {
@@ -486,23 +553,38 @@ function onViewAutoConf() {
 }
 
 function onTestAutoConf() {
-	if (_checkUri()) {
-	  try {
-		  CC["@leahscape.org/foxyproxy/autoconf;1"].createInstance().wrappedJSObject.testPAC(autoconfurl.value);
-      foxyproxy.alert(this, foxyproxy.getMessage("autoconfurl.test.success"));
-	  }
-	  catch (e) {
-	    foxyproxy.alert(this, foxyproxy.getMessage("autoconfurl.test.fail2", [e.message]));
-	  }
-	}
+  if (_checkUri()) {
+    let autoconfMessage = "";
+    try {
+      CC["@leahscape.org/foxyproxy/autoconf;1"].createInstance().
+        wrappedJSObject.testPAC(autoconfUrl.value);
+      if (autoconfMode.value === "pac") {
+        autoconfMessage = "autoconfurl.test.success";
+      } else {
+        autoconfMessage = "wpadurl.test.success";
+      }
+      foxyproxy.alert(this, foxyproxy.getMessage(autoconfMessage));
+    } catch (e) {
+      if (autoconfMode.value === "pac") {
+        autoconfMessage = "autoconfurl.test.fail2";
+      } else {
+        autoconfMessage = "wpadurl.test.fail";
+      } 
+      foxyproxy.alert(this, foxyproxy.getMessage(autoconfMessage, [e.message]));
+    }
+  }
 }
 
 function onAutoConfUrlInput() {
   // setAttribute("disabled", true) buggy in FF 1.5.0.4 for the way i've setup the cmd
   // so must use removeAttribute()
-	var b = document.getElementById("autoconf-broadcaster2");
-  if (autoconfurl.value.length > 0)
-    b.removeAttribute("disabled");
+  var b = document.getElementById("autoconf-broadcaster2");
+  // TODO: For some reason we need the second condition in order to disable the
+  // view and test button iff the dialog gets loaded AND the proxy mode is not
+  // "auto".
+  if (autoconfUrl.value.length > 0 && document.getElementById("mode").value ===
+    "auto")
+    b.setAttribute("disabled", "false");
   else
     b.setAttribute("disabled", "true");
 }
@@ -515,7 +597,7 @@ function onSelectAutoConf() {
   p.appendFilter(foxyproxy.getMessage("pac.files"), "*.pac");
   p.defaultExtension = "pac";
   if (p.show() != nsIFilePicker.returnCancel) {
-  	autoconfurl.value = foxyproxy.transformer(p.file, "uri-string");
+  	autoconfUrl.value = foxyproxy.transformer(p.file, "uri-string");
   	onAutoConfUrlInput();
   }
 }
