@@ -1,6 +1,6 @@
 /**
   FoxyProxy
-  Copyright (C) 2006-#%#% Eric H. Jung and LeahScape, Inc.
+  Copyright (C) 2006-#%#% Eric H. Jung and FoxyProxy, Inc.
   http://getfoxyproxy.org/
   eric.jung@yahoo.com
 
@@ -178,6 +178,34 @@ end-foxyproxy-simple !*/
 /*! end-foxyproxy-standard !*/
     }
     this.patternErrorNotification();
+    // TODO: Make that compatible with Thunderbird
+    try { 
+      if (gBrowser) {
+        gBrowser.addEventListener("DOMContentLoaded", foxyproxy.errorPageCheck,
+          false);
+      }
+    } catch(e) {}
+  },
+
+  parseHTML : function(doc, html) {
+    return Components.classes["@mozilla.org/feed-unescapehtml;1"].
+      getService(Components.interfaces.nsIScriptableUnescapeHTML).
+      parseFragment(html, false, null, doc.documentElement); 
+  },
+
+  errorPageCheck : function(e) {
+    var contDoc = window.content.document;
+    if (contDoc.documentURI.indexOf("about:neterror?e=proxyConnectFailure") ===
+      0) {
+      // Creating our additional list entry. We have to take this road here as
+      // creating the <li> element and assigning the content via innerHTML is
+      // not recommended. Furthermore, we cannot construct and append the <li>
+      // element ourselves properly due to i18n issues. Thus, we resort to
+      // parseFragment().
+      let liText = foxyproxy.fp.getMessage("foxyproxy.proxyservice");
+      contDoc.getElementById("errorLongDesc").firstChild.nextSibling.
+        appendChild(foxyproxy.parseHTML(contDoc, liText));
+    }
   },
 
   patternErrorNotification : function() {
@@ -439,7 +467,7 @@ end-foxyproxy-simple !*/
     // Update view if it's open
     var optionsDlg = foxyproxy._getOptionsDlg();
     optionsDlg && optionsDlg._updateView(false, updateLogView); // don't write settings here because optionsDlg mayn't be open
-    writeSettings && this.fp.writeSettings();
+    writeSettings && this.fp.writeSettingsAsync();
   },
 
   _getOptionsDlg : function() {
@@ -509,16 +537,17 @@ end-foxyproxy-simple !*/
     var listen = this.fp.mode != "disabled" && this.fp.autoadd.enabled;
     var appcontent = document.getElementById("appcontent");
     if (appcontent) {
-      appcontent.removeEventListener("load", this.onPageLoad, true); // safety
+      // Safety. We use here and in the following |foxyproxy| and not |this| as
+      // this makes it easier to remove the event listener on unload again.
+      appcontent.removeEventListener("load", foxyproxy.onPageLoad, true); 
       if (listen) {
-        appcontent.addEventListener("load", this.onPageLoad, true);
+        appcontent.addEventListener("load", foxyproxy.onPageLoad, true);
       }
       else {
-        appcontent.removeEventListener("load", this.onPageLoad, true);
+        appcontent.removeEventListener("load", foxyproxy.onPageLoad, true);
       }
     }
   },
-    
 
   ///////////////// icons \\\\\\\\\\\\\\\\\\\\\
   svgIcons : {
@@ -801,14 +830,16 @@ end-foxyproxy-simple !*/
           // Enable/disable checkbox for each proxy.
           // Don't provide enable/disable to lastresort proxy.
           !p.lastresort && _createCheckMenuItem(sbm,
-            curProxy + "enabled=!" + curProxy + "enabled;",
+            curProxy + "enabled=!" + curProxy +
+            "enabled;foxyproxy.fp.writeSettingsAsync();",
             p.enabled,
             this.fp.getMessage("foxyproxy.enabled.accesskey"),
             this.fp.getMessage("foxyproxy.enabled.label"),
             this.fp.getMessage("foxyproxy.enabled.tooltip"));
 
           _createCheckMenuItem(sbm,
-            curProxy + "animatedIcons=!" + curProxy + "animatedIcons;",
+            curProxy + "animatedIcons=!" + curProxy +
+            "animatedIcons;foxyproxy.fp.writeSettingsAsync();",
             p.animatedIcons,
             this.fp.getMessage("foxyproxy.animatedicons.accesskey"),
             this.fp.getMessage("foxyproxy.animatedicons.label"),
@@ -837,7 +868,7 @@ end-foxyproxy-simple !*/
               var m = this.fp.proxies.item(i).matches[j];
               var curMatch = curProxy + "matches[" + j + "].";
               _createCheckMenuItem(pmp,
-                curMatch + "enabled=!" + curMatch + "enabled;foxyproxy.fp.writeSettings();",
+                curMatch + "enabled=!" + curMatch + "enabled;foxyproxy.fp.writeSettingsAsync();",
                 m.enabled,
                 m.pattern.substring(0, 1),
                 m.pattern,
@@ -1122,8 +1153,17 @@ end-foxyproxy-simple !*/
 
 window.addEventListener("load", function(e) { foxyproxy.onLoad(e); }, false);
 window.addEventListener("unload", function(e) {
-  document.getElementById("appcontent") && document.getElementById("appcontent").removeEventListener("load", this.onPageLoad, true);
-  var obSvc = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  document.getElementById("appcontent") &&
+    document.getElementById("appcontent").removeEventListener("load", foxyproxy.
+    onPageLoad, true);
+  try {
+    if (gBrowser) {
+      gBrowser.removeEventListener("DOMContentLoaded", foxyproxy.errorPageCheck,
+        false);
+    }
+  } catch(e) {}
+  var obSvc = Components.classes["@mozilla.org/observer-service;1"].
+    getService(Components.interfaces.nsIObserverService);
   for (var i in foxyproxy.notes) {
     obSvc.removeObserver(foxyproxy, foxyproxy.notes[i]);
   }
